# API Docs - v5.2.1

!!! Info "Tested Siddhi Core version: *<a target="_blank" href="http://siddhi.io/en/v5.1/docs/query-guide/">5.1.14</a>*"
    It could also support other Siddhi Core minor versions.

## Sinkmapper

### xml *<a target="_blank" href="http://siddhi.io/en/v5.1/docs/query-guide/#sink-mapper">(Sink Mapper)</a>*
<p></p>
<p style="word-wrap: break-word;margin: 0;">This mapper converts Siddhi output events to XML before they are published via transports that publish in XML format. Users can either send a pre-defined XML format or a custom XML message containing event data.</p>
<p></p>
<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>

```
@sink(..., @map(type="xml", validate.xml="<BOOL>", enclosing.element="<STRING>")
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">validate.xml</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">This parameter specifies whether the XML messages generated should be validated or not. If this parameter is set to true, messages that do not adhere to proper XML standards are dropped. </p></td>
        <td style="vertical-align: top">false</td>
        <td style="vertical-align: top">BOOL</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">enclosing.element</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">When an enclosing element is specified, the child elements (e.g., the immediate child elements) of that element are considered as events. This is useful when you need to send multiple events in a single XML message. When an enclosing element is not specified, one XML message per every event will be emitted without enclosing.</p></td>
        <td style="vertical-align: top">None in custom mapping and &lt;events&gt; in default mapping</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@sink(type='inMemory', topic='stock', @map(type='xml'))
define stream FooStream (symbol string, price float, volume long);

```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above configuration will do a default XML input mapping which will generate below output<br>&lt;events&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;event&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;symbol&gt;WSO2&lt;/symbol&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;price&gt;55.6&lt;/price&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;volume&gt;100&lt;/volume&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/event&gt;<br>&lt;/events&gt;<br></p>
<p></p>
<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
@sink(type='inMemory', topic='{{symbol}}', @map(type='xml', enclosing.element='<portfolio>', validate.xml='true', @payload( "<StockData><Symbol>{{symbol}}</Symbol><Price>{{price}}</Price></StockData>")))
define stream BarStream (symbol string, price float, volume long);
```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above configuration will perform a custom XML mapping. Inside @payload you can specify the custom template that you want to send the messages out and addd placeholders to places where you need to add event attributes.Above config will produce below output XML message<br>&lt;portfolio&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;StockData&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;Symbol&gt;WSO2&lt;/Symbol&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;Price&gt;55.6&lt;/Price&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/StockData&gt;<br>&lt;/portfolio&gt;</p>
<p></p>
## Sourcemapper

### xml *<a target="_blank" href="http://siddhi.io/en/v5.1/docs/query-guide/#source-mapper">(Source Mapper)</a>*
<p></p>
<p style="word-wrap: break-word;margin: 0;">This mapper converts XML input to Siddhi event. Transports which accepts XML messages can utilize this extension to convert the incoming XML message to Siddhi event. Users can either send a pre-defined XML format where event conversion will happen without any configs or can use xpath to map from a custom XML message.</p>
<p></p>
<span id="syntax" class="md-typeset" style="display: block; font-weight: bold;">Syntax</span>

```
@source(..., @map(type="xml", namespaces="<STRING>", enclosing.element="<STRING>", enclosing.element.as.event="<BOOL>", fail.on.missing.attribute="<BOOL>", enable.streaming.xml.content="<BOOL>", extract.leaf.node.data="<BOOL>")
```

<span id="query-parameters" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">QUERY PARAMETERS</span>
<table>
    <tr>
        <th>Name</th>
        <th style="min-width: 20em">Description</th>
        <th>Default Value</th>
        <th>Possible Data Types</th>
        <th>Optional</th>
        <th>Dynamic</th>
    </tr>
    <tr>
        <td style="vertical-align: top">namespaces</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">Used to provide namespaces used in the incoming XML message beforehand to configure xpath expressions. User can provide a comma separated list. If these are not provided xpath evaluations will fail</p></td>
        <td style="vertical-align: top">None</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">enclosing.element</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">Used when custom mapping is enabled to specify the XPath in case of sending multiple events (if XPath consist of multiple elements, or consists of child elements.) using the same XML content or when the event is not in root node.<br>XML mapper will treat the child element/s of given enclosing element as event/s, when <code>enclosing.element.as.event</code> is set to <code>false</code>, and execute xpath expressions available in the attribute mappings on child elements.<br>&nbsp;If enclosing.element is not provided XPaths in the attribute mappings will be evaluated with respect to root element.<br>When enclosing element is used and custom mapping is enabled, the attribute mapping XPaths should be relative to child elements</p></td>
        <td style="vertical-align: top">Root element</td>
        <td style="vertical-align: top">STRING</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">enclosing.element.as.event</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">This can either have value true or false. XML mapper will treat the child element/s of given enclosing element as event/s, when <code>enclosing.element.as.event</code> is set to <code>false</code>, and attribute mapping should be defined with with respect to the Xpath element's child elements.<br>&nbsp;When <code>enclosing.element.as.event</code> is set to <code>true</code>, the elements (NOT child elements) retrieved from XPath itself will be treated as the event/s and attribute mapping should be defined with with respect to the Xpath element.<br>When <code>enable.streaming.xml</code> is set to <code>true</code> the <code>enclosing.element.as.event</code> value will be set to <code>true</code> by default</p></td>
        <td style="vertical-align: top">False</td>
        <td style="vertical-align: top">BOOL</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">fail.on.missing.attribute</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">This can either have value true or false. By default it will be true. This attribute allows user to handle unknown attributes. By default if an xpath execution fails or returns null DAS will drop that message. However setting this property to false will prompt DAS to send and event with null value to Siddhi where user can handle it accordingly(ie. Assign a default value)</p></td>
        <td style="vertical-align: top">True</td>
        <td style="vertical-align: top">BOOL</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">enable.streaming.xml.content</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">This will be used when the XML content is streamed without sending the XML element/ event as a whole. eg: When streaming XML file line by line.</p></td>
        <td style="vertical-align: top">false</td>
        <td style="vertical-align: top">BOOL</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
    <tr>
        <td style="vertical-align: top">extract.leaf.node.data</td>
        <td style="vertical-align: top; word-wrap: break-word"><p style="word-wrap: break-word;margin: 0;">This parameter will enable the event to contain the child element values.<br>This can be used when the given XPath node contains several child elements and also when the child elements have child elements as well.<br>If there are multiple child elements, the event count for one node will get multiplied by the number of child nodes.</p></td>
        <td style="vertical-align: top">false</td>
        <td style="vertical-align: top">BOOL</td>
        <td style="vertical-align: top">Yes</td>
        <td style="vertical-align: top">No</td>
    </tr>
</table>

<span id="examples" class="md-typeset" style="display: block; font-weight: bold;">Examples</span>
<span id="example-1" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 1</span>
```
@source(type='inMemory', topic='stock', @map(type='xml'))
define stream FooStream (symbol string, price float, volume long);

```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above configuration will do a default XML input mapping. Expected input will look like below.&lt;events&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;event&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;symbol&gt;WSO2&lt;/symbol&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;price&gt;55.6&lt;/price&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;volume&gt;100&lt;/volume&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/event&gt;<br>&lt;/events&gt;<br></p>
<p></p>
<span id="example-2" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 2</span>
```
@map(type='xml', enclosing.element="/events/wrapper/event", 
     enclosing.element.as.event="true", 
     @attributes(volume = "volume", price = "price", symbol = "symbol"))
define stream FooStream (symbol string, price float, volume long);

```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above configuration will do a custom mapping and and get the &lt;event&gt; element as a whole event in the given XPath. The attribute mapping has to be done with respect to the element mapped in the XPath. Expected input will look like below&lt;events&gt;&lt;wrapper&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;event&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;symbol&gt;WSO2&lt;/symbol&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;price&gt;55.6&lt;/price&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;volume&gt;100&lt;/volume&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/event&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;event&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;symbol&gt;WSO2&lt;/symbol&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;price&gt;55.6&lt;/price&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;volume&gt;100&lt;/volume&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/event&gt;<br>&lt;/wrapper&gt;&lt;/events&gt;<br></p>
<p></p>
<span id="example-3" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 3</span>
```
@map(type='xml', enclosing.element='/events/event',
      @attributes(symbol='symbol', price='price', volume='volume'))
define stream FooStream (symbol string, price float, volume long);

```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above configuration will do a custom mapping and and get the &lt;event&gt; element as a whole event in the given XPath. The attribute mapping has to be done with respect to the element mapped in the XPath. Expected input will look like below.&lt;events&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;event&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;symbol&gt;WSO2&lt;/symbol&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;price&gt;55.6&lt;/price&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;volume&gt;100&lt;/volume&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/event&gt;<br>&lt;/events&gt;<br></p>
<p></p>
<span id="example-4" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 4</span>
```
@source(type='inMemory', topic='stock', 
  @map(type='xml', namespaces = "dt=urn:schemas-microsoft-com:datatypes", 
       enclosing.element="//portfolio", 
       @attributes(symbol = "company/symbol", price = "price", 
                   volume = "volume")))
define stream FooStream (symbol string, price float, volume long);
```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above configuration will perform a custom XML mapping. In the custom mapping user can add xpath expressions representing each event attribute using @attribute annotation. Expected input will look like below.<br>&lt;portfolio xmlns:dt="urn:schemas-microsoft-com:datatypes"&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;stock exchange="nasdaq"&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;volume&gt;100&lt;/volume&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;company&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;symbol&gt;WSO2&lt;/symbol&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;/company&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;price dt:type="number"&gt;55.6&lt;/price&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&lt;/stock&gt;<br>&lt;/portfolio&gt;</p>
<p></p>
<span id="example-5" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 5</span>
```
@map(type='xml', 
      enclosing.element="/root/child",
      enable.streaming.xml.content="true",
      enclosing.element.as.event="true",
      extract.leaf.node.data="true",
      fail.on.missing.attribute="false",
      @attributes(id = "/child/@id", timestamp = "/child/@timestamp", 
                  key = "/child/detail/@key", 
                  value = "/child/detail/@value"))
define stream FooStream (id string, timestamp string, key string, value string);

```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above configuration will do a custom mapping and and get the &lt;child&gt; element as a whole event in the given XPath when the XML content received in a steaming manner (eg: when a file is read line by line and sent to the XML mapper to map).<br>The attribute mapping has to be done with respect to the element mapped in the XPath which is &lt;child&gt;<br>In here, the leaf node data is mapped to the &lt;child&gt; event as well. <br>Expected input will look like below.<br>&lt;root&gt;<br>&nbsp;&nbsp;&nbsp;&lt;bounds minlat="53.4281" minlon="-2.4142" maxlat="54.0097" maxlon="-0.9762"/&gt;<br>&nbsp;&nbsp;&nbsp;&lt;child id="413229" timestamp="2014-09-10T14:12:48Z"/&gt;<br>&nbsp;&nbsp;&nbsp;&lt;child id="414427" timestamp="2018-01-24T23:16:10Z"/&gt;<br>&nbsp;&nbsp;&nbsp;&lt;child id="673959" timestamp="2019-10-20T12:07:13Z"&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;extra id="1234" timestamp="2014-09-11T10:36:37Z"/&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;detail key="company" value="wso2"/&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;extra id="0987" timestamp="2014-09-11T10:36:37Z"/&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;detail key="country" value="Sri Lanka"/&gt;<br>&nbsp;&nbsp;&nbsp;&lt;/child&gt;<br>&nbsp;&nbsp;&nbsp;.<br>&nbsp;&nbsp;&nbsp;.<br>&lt;/root&gt;<br></p>
<p></p>
<span id="example-6" class="md-typeset" style="display: block; color: rgba(0, 0, 0, 0.54); font-size: 12.8px; font-weight: bold;">EXAMPLE 6</span>
```
@map(type='xml', 
      enclosing.element="/root/child",
      enable.streaming.xml.content="true",
      enclosing.element.as.event="true",
      fail.on.missing.attribute="false",
      @attributes(id = "/child/@id", timestamp = "/child/@timestamp"))
define stream FooStream (id string, timestamp string, key string, value string);

```
<p></p>
<p style="word-wrap: break-word;margin: 0;">Above configuration will do a custom mapping and and get the &lt;child&gt; element as a whole event in the given XPath when the XML content received in a steaming manner (eg: when a file is read line by line and sent to the XML mapper to map).<br>The attribute mapping has to be done with respect to the element mapped in the XPath which is &lt;child&gt;<br>Expected input will look like below.<br>&lt;root&gt;<br>&nbsp;&nbsp;&nbsp;&lt;bounds minlat="53.4281" minlon="-2.4142" maxlat="54.0097" maxlon="-0.9762"/&gt;<br>&nbsp;&nbsp;&nbsp;&lt;child id="413229" timestamp="2014-09-10T14:12:48Z"/&gt;<br>&nbsp;&nbsp;&nbsp;&lt;child id="414427" timestamp="2018-01-24T23:16:10Z"/&gt;<br>&nbsp;&nbsp;&nbsp;&lt;child id="673959" timestamp="2019-10-20T12:07:13Z"&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;extra id="1234" timestamp="2014-09-11T10:36:37Z"/&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;detail key="company" value="wso2"/&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;extra id="0987" timestamp="2014-09-11T10:36:37Z"/&gt;<br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&lt;detail key="country" value="Sri Lanka"/&gt;<br>&nbsp;&nbsp;&nbsp;&lt;/child&gt;<br>&nbsp;&nbsp;&nbsp;.<br>&nbsp;&nbsp;&nbsp;.<br>&lt;/root&gt;<br></p>
<p></p>
